// udStFiLrTabDelimited
// J. J. Weimer
// History: $Revision: 516 $ on $Date: 2008-02-11 12:21:50 -0500 (Mon, 11 Feb 2008) $
//
// Procedure files to read a tab delimited file and store it
//
//
// Access to Package
//
// -> udStFiLrTabDelimited(udFL)
//		STRUCTURE udFLStandardStructure &udFL
//
// see the structure definition and User's Manual

#pragma rtGlobals=1
#pragma IgorVersion=6.03
#pragma version=1.10

#include "udFLStandardFunctions", version >= 1.10

// **********************************************
// Globals and Localizations

Static Structure tabdelimitedglobals
	string version
EndStructure

Static Function Inittabdelimitedglobals(tbd)
	STRUCT tabdelimitedglobals &tbd

	tbd.version="1.10"
End

Static Function udFLTabDInititalize(udFL)
	STRUCT udFLStandardStructure &udFL
	
	udFLZeroStructure(udFL)
	udFLTabDQuery(udFL)
	return 0
end

Static Function udFLTabDQuery(udFL)
	STRUCT udFLStandardStructure &udFL
		
	udFL.mimetype = "TEXT"
	udFL.extensions = ".txt;.dat;"
	udFL.procModes = "Auto;Manual;"
	udFL.itemsType = 4			// returns waves
	udFL.itemsDim[p] = p*0
	udFL.itemsDim[0]=-1		// waves are nominally unscaled numeric
	udFL.setNames=(2+8+32)		// filename+number or prefix+number or loader-defined supported
	udFL.userData = "Prefix+Column#;"
	udFLTabDInitPlugIns(udFL)
	return 0	
end

Static Function udFLTabDInitPlugIns(udFL)
	STRUCT udFLStandardStructure &udFL
//
//	if(udFLCheckForPlugIn("SpXZeigR")==1)
//		udFL.procModes += "--;Auto Interpolate XY..Z->Y Z;Auto Interpolate XY XZ .. ->Y Z;"
//	endif
	
	return 0
end

// **********************************************
// File Loader

Function udStFiLrTabDelimited(udFL)
	STRUCT udFLStandardStructure &udFL

	variable refNum, ic, nt
	string pathStr, fileStr
	
	switch(udFL.userCtrl)
		case -1:		// initialize		
			udFLTabDInititalize(udFL)		
			return 0
		case 0:		// query
			udFLTabDQuery(udFL)
			return 0
		case 1:		// auto
			if (strlen(udFL.fileList)==0)
				udFLSetError(udFL,"No files in the selection list!",-1)
				return -1
			endif
			break
		case 2:		// manual
			Open/C="TEXT"/D/R/M="Select the tab-delimited file" refNum
			if (strlen(S_filename)!=0)
				udFL.filelist = S_filename + ";"
			else
				udFLSetError(udFL,"No file selected",-1)
				return -1
			endif
			break
	endswitch
	if (udFL.reportCtrl >= 1)
		print "LOADING TAB-DELIMITED DATA"
		print "... assuring file paths"
	endif
	if (udFLAssureFilePaths(udFL)<0)
		return -1
	endif
	if (udFL.reportCtrl >= 1)
		print "... setting data folder location"
	endif
	string cdf = GetDataFolder(1)
	udFLSetDataFolder(udFL)
	if (strlen(udFL.pathStr)!=0)
		pathStr = udFL.pathStr
		NewPath/O/Q tmpTabDdir, pathStr
	endif
	if (udFL.reportCtrl >= 1)
		print "... loading file(s)"
	endif
	nt = ItemsInList(udFL.fileList)
	for (ic=0;ic<nt;ic+=1)
		fileStr=StringFromList(ic,udFL.fileList)
		if (udFL.reportCtrl < 0)
			if (strlen(udFL.pathStr)==0)
				LoadWave/Q/N=TDwave/J/K=0 fileStr
			else
				LoadWave/Q/N=TDwave/J/K=0/P=tmpTabDdir fileStr
			endif
		else
			if (strlen(udFL.pathStr)==0)
				LoadWave/A/J/K=0 fileStr
			else
				LoadWave/A/J/K=0/P=tmpTabDdir fileStr
			endif
		endif
		
		// user data requests scaled wave from (at least) two unscaled waves
		
		variable xs=0, xe=0, ni=0, jc=0
		string xwave, ywave, wList = "", theStr = "", itHas = ""
		string columnList = ""

		// loop over all extra (userData) processing requests
		
		do
		
			theStr = udFLStripUserDataString(udFL)
			if (strlen(theStr)==0)
				break
			endif
			strswitch(theStr)
				case "s0":
					ni = ItemsInList(S_wavenames)
					if (ni<2)
						udFLSetError(udFL,"Could not scale only one wave",0)
						break
					endif
					if (udFL.reportCtrl >= 1)
						print "... scaling all wave(s) based on data of equal increment in first column"
					endif
					xwave = StringFromList(0,S_wavenames)
					wave nwave=$xwave
					xs = nwave[0]
					xe = nwave[numpnts(nwave)-1]
					killwaves/Z nwave
					for (jc=1;jc<ni;jc+=1)
						ywave = StringFromList(jc,S_wavenames)
						wave nwave = $ywave
						SetScale/I x, xs, xe, nwave
						wList += ywave + ";"
					endfor
					S_wavenames = wList
					break
				case "sp":
					ni = ItemsInList(S_wavenames)
					if (ni<2)
						udFLSetError(udFL,"Could not scale only one wave",0)
						break
					endif
					if (mod(ni,2)!=0)
						udFLSetError(udFL,"Could not scale as wave pairs with odd number of columns",0)
						break
					endif
					if (udFL.reportCtrl >= 1)
						print "... scaling wave(s) as paired (x,y) columns"
					endif
					for (jc=0;jc<ni;jc+=2)
						xwave = StringFromList(jc,S_wavenames)
						wave nwave=$xwave
						xs = nwave[0]
						xe = nwave[numpnts(nwave)-1]
						killwaves/Z nwave
						ywave = StringFromList(jc+1,S_wavenames)
						wave nwave = $ywave
						SetScale/I x, xs, xe, nwave
						wList += ywave + ";"
					endfor
					S_wavenames = wList
					break
				default:
					SplitString/E="\[(.+)\]" theStr, itHas
					break
			endswitch
				
		while(1)
		
		if (strlen(itHas)!=0)
			ni = ItemsInList(S_wavenames)
			if (ni>2)
				if (udFL.reportCtrl >= 1)
					print "... selecting wave(s) based on input: ", itHas
				endif
				theStr = ""
				xe = ItemsInList(itHas,",")
				for (jc=0;jc<xe;jc+=1)
					theStr += ExpandString(StringFromList(jc,itHas,","),sep=";")
				endfor
				itHas = ""
				xs = 0
				xe = str2num(StringFromList(xs,theStr))
				for(jc=1;jc<(ni+1);jc+=1)
					if (jc != xe)
						xwave = StringFromList(jc-1,S_wavenames)
						wave nwave = $xwave
						killwaves/Z nwave
					else
						itHas += StringFromList(jc-1,S_wavenames) + ";"
						xs += 1
						xe = str2num(StringFromList(xs,theStr))
					endif
				endfor
				S_wavenames = itHas
			else
				udFLSetError(udFL,"Cannot select with only one wave input",0)
			endif
		endif					
		
		columnList = S_wavenames
		
		// possibly rename waves
		
		switch(udFL.setNames)
			case 1:
				S_wavenames=udFLRenameInput(S_wavenames,"TDwave")
				break
			case 2:
				udFL.namePrefix = ReplaceString(" ",ParseFilePath(3,fileStr,":",0,0),"")
				if (udFL.reportCtrl >= 1)
					print "... renaming wave(s) by filename ", udFL.namePrefix, " + number"
				endif
				S_wavenames=udFLRenameInput(S_waveNames,udFL.namePrefix)
				break
			case 8:
				if (udFL.reportCtrl >= 1)
					print "... renaming wave(s) by prefix ", udFL.namePrefix, " + number"
				endif
				S_wavenames=udFLRenameInput(S_waveNames,udFL.namePrefix)
				break
			case 32:
				if (udFL.reportCtrl >= 1)
					print "... renaming wave(s) by prefix ", udFL.namePrefix, " + column number"
				endif
				ni = ItemsInList(columnList)
				S_wavenames = ""
				for (jc=0;jc<ni;jc+=1)
					theStr = StringFromList(jc,columnList)
					itHas = ReplaceString("wave",theStr,"")
					S_wavenames+=udFLRenameInput(theStr,(udFL.namePrefix+itHas))
				endfor
				break
		endswitch
		udFL.itLst.WAVES += S_wavenames
	endfor
	udFL.itemsCount = ItemsInList(udFL.itLst.WAVES)
	SetDataFolder $cdf
	
	if (udFL.reportCtrl == 2)
		print "Structure @ Loader:", udFL
	endif
	return 0
end

Static Function/S ExpandString(theStr, [sep])
	string theStr, sep
	
	string xs, xe
	variable ns, ne, ic

	if(ParamIsDefault(sep))
		sep = ","
	endif
	
	SplitString/E="(.+)-(.+)" theStr, xs,xe
	
	if (strlen(xs)==0)
		return (theStr + sep)
	endif
	ns = str2num(xs)
	ne = str2num(xe)
	
	theStr = ""
	for(ic=ns;ic<(ne+1);ic+=1)
		theStr += num2str(ic) + sep
	endfor
	return theStr
end